/***************************************************************************
* 
*  Irradiancemètre
*  Utilisation d'un panneau de 5W en court circuit et mesure de Icc via un INA226
*  Icc est proportionnel a l'irradiance
*  Etalonnage en appuyant sur BP pour accéder au mode où l'ajustable multitour permet de régler l'irradaiance mesurée
*  En relachant BP, la valeur de Icc correspondant à 1000W/m2 est stockée en EEPPROM
*
***************************************************************************/
/*         cablage
 * 
 * INA226:
 * VCC +5V
 * GND GND
 * SDA A4
 * SCL A5
 * ALE nc
 * UBS nc
 * IN- PV1
 * IN+ PV2
 * 
 * 
 * LCD:
 * 1 GND
 * 2 +5V
 * 3 Curseur P=10k
 * 4 D12
 * 5 GND
 * 6 D11
 * 7 nc
 * 8 nc
 * 9 nc
 * 10 nc
 * 11 D5
 * 12 D4
 * 13 D3
 * 14 D2
 * 15 nc res vers +5V si rétroéclairage
 * 16 nc GND si retroeclairage
 * 
 * BP:
 * SW1 D6
 * SW2 GND
 * 
 * Pot réglage:
 * Curseur A0
 * GND/+5V
 * 
 */
#include <Wire.h>
#include <INA226_WE.h>
#include <LiquidCrystal.h>
#include <EEPROM.h>

const int rs = 12, en = 11, d4 = 5, d5 = 4, d6 = 3, d7 = 2;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

#define I2C_ADDRESS 0x40
#define BP 6   // BP sur D6
const char * ver = "1.0"; // version logicielle
int sensorPin = A0;    // l'ajustable d'étalonnage est connecté sur A0

INA226_WE ina226 = INA226_WE(I2C_ADDRESS);

int ICC1000=330;  // étalonage par défaut de l'intensité en mA à 1000W/m2 du panneau utilisé

void setup()
{
  pinMode(BP, INPUT_PULLUP); // BP étalonnage avec pullup
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.print("(c) silicium81");
  lcd.setCursor(0, 1);
  lcd.print("version:");
  lcd.print(ver);
  delay(2000);
  if (EEPROM.read(0)!=85)  // l'EEPROM est vierge ou corrompue, on y place une valeur de ICC1000 par défaut
  { 
    EEPROM.write(0, 85); // on place 85 à l'adresse 0 pour indiquer un contenu valide de ICC1000 aux adresses 1 et 2 de l'EEPROM
    EEPROM.write(1, (ICC1000&0b11111111));
    EEPROM.write(2, (ICC1000&0b1111111100000000)>>8);
  }
  
  ICC1000 = EEPROM.read(1)+ 256*EEPROM.read(2);  // lecture de la valeur de ICC1000 dans l'EEPROM
  lcd.setCursor(0, 0);
  lcd.print("Icc actual     ");
  lcd.setCursor(0, 1);
  lcd.print(ICC1000); lcd.print(" mA           ");
  delay(2000);
  Serial.begin(9600);
  Wire.begin();
  ina226.init();
  ina226.setAverage(AVERAGE_256); // choose mode and uncomment for change of default
}

void loop()
{
  float current_mA = 0.0;
  int irradiance;
  int sensorValue=0;
  if (!digitalRead(BP)) // demande d'étalonnage
  {
    do
    {
      ina226.readAndClearFlags();
      current_mA = ina226.getCurrent_mA();
      lcd.setCursor(0, 0);
      lcd.print("Icc1000: ");
      lcd.print(sensorValue); lcd.print("    ");
      lcd.setCursor(0, 1);
      sensorValue=analogRead(sensorPin);
      irradiance=1000*current_mA/sensorValue;
      lcd.print(irradiance);
      lcd.print(" W/m2          ");
      delay(100);
    }
    while (!digitalRead(BP));
    ICC1000= sensorValue; 
    EEPROM.write(0, 85);
    EEPROM.write(1, (ICC1000&0b11111111));
    EEPROM.write(2, (ICC1000&0b1111111100000000)>>8);
    lcd.setCursor(0, 0);
    lcd.print("Icc actualised     ");
    lcd.setCursor(0, 1);
    lcd.print(ICC1000); lcd.print(" mA           ");
    delay(2000);
  }
  
  ina226.readAndClearFlags();
  current_mA = ina226.getCurrent_mA();

//** affichage sur le lcd
  lcd.setCursor(0, 0);
  lcd.print("Irradiance:       ");
  lcd.setCursor(0, 1);
  irradiance=1000*current_mA/ICC1000;
  lcd.print(irradiance);
  lcd.print(" W/m2          ");

//** sortie série
  Serial.print("Irradiance: ");
  Serial.print(irradiance);
  Serial.println(" W/m2");
  
  delay(1000);
}
